/*************************************************
 * Some common stuff: debug, r/w files, etc.
 ************************************************/

/* ********************************************* *
 * Localisation
 * ********************************************* */

var ILO_BUNDLE = null;

/* ********************************************* *
 * Constants, preferences, observer.
 * ********************************************* */
const IMGLIKEOPERA_GUID = "{b1d008f4-5fcd-4a34-8506-b3143b6caf94}";

const ILO_PREF_SERVICE = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefService);

const ILO_PREF_BRANCH = ILO_PREF_SERVICE.getBranch("imglikeopera.");

const ILO_OBSERV_SERVICE = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);

/* ********************************************* *
 * dEbUG
 * ********************************************* */
const ILO_CONSOLE_SERVICE = Components.classes["@mozilla.org/consoleservice;1"]
        .getService(Components.interfaces.nsIConsoleService);

function idump(msg) {
    msg = "[ilo_debug]: " + msg + "\n";
    ILO_CONSOLE_SERVICE.logStringMessage(msg);
    dump(msg);
}

/* ********************************************* *
 *
 * FileSystemStuff
 *
 * ********************************************* *
 *  Open flags
 *    const MODE_RDONLY   = 0x01;
 *    const MODE_WRONLY   = 0x02;
 *    const MODE_RDWR     = 0x04;
 *    const MODE_CREATE   = 0x08;
 *    const MODE_APPEND   = 0x10;
 *    const MODE_TRUNCATE = 0x20;
 *    const MODE_SYNC     = 0x40;
 *    const MODE_EXCL     = 0x80;
 *
 *  File modes ....
 *  'mode' is currently only applicable on UNIX platforms.
 *  The 'mode' argument may be ignored by PR_Open on other platforms.
 *
 *    00400   Read by owner.
 *    00200   Write by owner.
 *    00100   Execute (search if a directory) by owner.
 *    00040   Read by group.
 *    00020   Write by group.
 *    00010   Execute by group.
 *    00004   Read by others.
 *    00002   Write by others
 *    00001   Execute by others.
 * ********************************************* */

/* ********************************************* *
 * Import/Export filters
 * ********************************************* */

//~ Fill filters tree.
function iloFillFiltersList(patterns, overwriteCurrentList)
{
    if (overwriteCurrentList)
		iloRemoveFilter("all");
    
    var patt;
    for (var i = 0; i < patterns.length; i++)
    {
        patt = patterns[i].split(" ");
        patt[1] = patt[1][0];
        if (patt.length == 2 && patt[1]>0 && patt[1]<5)
            iloTreeAddItem(patt);
    }
}

//~ Imports filters from disc.
function iloFiltersImportList() {
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
	var stream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
	var streamIO = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
	var overwriteCurrentList = false;
	var input;
	var inputArray;
	var validFile = false;
	
	fp.init(window, "Select a File", fp.modeOpen);
	fp.appendFilters(fp.filterText);

	if (fp.show() != fp.returnCancel)
    {
		stream.init(fp.file, 0x01, 0444, null);
		streamIO.init(stream);
		input = streamIO.read(stream.available());
		streamIO.close();
		stream.close();
		
		//~ unix + mac + dos environment-compatible
		//~ first: whole match -- second: backref-1 -- etc..
        linebreak = input.match(/(?:\[ImgLikeOpera\])(((\n+)|(\r+))+)/m)[1];
        
		inputArray = input.split(linebreak);
        
		var headerRe = /\[ImgLikeOpera\]/; //~ tests if the first line is ILO's header
		if (headerRe.test(inputArray[0]))
        {
			inputArray.shift();
            overwriteCurrentList = confirm(ILO_BUNDLE.getString("overwriteCurrentListConfirm"));
            iloFillFiltersList(inputArray, overwriteCurrentList);
        }
		else 
			alert(ILO_BUNDLE.getString("fileNotValid"));
	}
}

//~ Exports the current list of filters to a file on disc.
function iloFiltersExportList()
{
	var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
	var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);

	fp.init(window, "Select a File", fp.modeSave);
	fp.appendFilters(fp.filterText);

	if (fp.show() != fp.returnCancel)
    {
		if (fp.file.exists())
			fp.file.remove(true);
		fp.file.create(fp.file.NORMAL_FILE_TYPE, 0666);
        
		stream.init(fp.file, 0x02, 0x200, null);
		stream.write("[ImgLikeOpera]\n", 15);
		
		patterns = iloTreeMakePattern().split("  ");
		
        for (var i = 0; i < patterns.length-1; i++)
            stream.write(patterns[i]+"\n", patterns[i].length+1);
        
        stream.close();
	}
}

/* ********************************************* *
 * userChrome.css: adding our @import
 * ********************************************* */

//~ Returns a nsIFile for the specified file in the profile chrome directory
function iloGetUserChromeFile(fileName)
{
    var NSIFILE = Components.interfaces.nsIFile;
    var dirLocator = Components.classes["@mozilla.org/file/directory_service;1"]
                               .getService(Components.interfaces.nsIProperties);
    var userChromePath = dirLocator.get("UChrm", NSIFILE).path;

    var file = Components.classes["@mozilla.org/file/local;1"]
                         .createInstance(Components.interfaces.nsILocalFile);

    file.initWithPath(userChromePath);
    file.append(fileName);

    return file;
}

//~ Returns the contents of the specified file in the profile chrome directory
function iloReadUserChromeFile(fileName) {
    var fileContents = "";
    var file = iloGetUserChromeFile(fileName);

    if(file.exists()) {
        var ioFlags = 0x01;

        //~ Get an input stream
        var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
                           .createInstance( Components.interfaces.nsIFileInputStream);
        is.init(file, ioFlags, 0, is.CLOSE_ON_EOF);
        var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
                            .createInstance( Components.interfaces.nsIScriptableInputStream);
        sis.init(is);

        //~ Read the file in
        while(sis.available() > 0)
            fileContents += sis.read(sis.available());

        //~ Close streams
        is.close();
        sis.close();
    }

    return fileContents;
}

//~ Writes the specified contents into the specified file in the profile chrome directory
function iloWriteUserChromeFile(fileName, fileContents) {
    var file = iloGetUserChromeFile(fileName);
    var ioFlags = 0x02 | 0x08 | 0x20;
    var perm = 0644;

    var os = Components.classes["@mozilla.org/network/file-output-stream;1"]
                       .createInstance( Components.interfaces.nsIFileOutputStream);
    os.init(file, ioFlags, perm, 0);

    var result = os.write(fileContents, fileContents.length);
    os.close();

    return result;
}

//~ Checks the if the user stylesheet already contains the import statement
function iloUserStyleSheetHasImport() {
    var fileContents = iloReadUserChromeFile('userContent.css');
    var re = new RegExp("^[ \t]*@import.*bonus.ilo.css", "m");

    return re.test(fileContents);
}

//~ Adds a CSS import statement for the imglikeopera stylesheet
function iloAddImportToUserStylesheet(fileName) {
    var importStr = "@import url(bonus.ilo.css);"

    var fileContents = iloReadUserChromeFile('userContent.css');
    var re = new RegExp("^[ \t]*@import.*bonus.ilo.css", "m");

    if(re.test(fileContents))
        return true;

    fileContents = importStr + "\n" + fileContents;

    var ret = iloWriteUserChromeFile(fileName, fileContents);
    return (ret == fileContents.length);
}

//~ Removes the CSS import statement for the imglikeopera stylesheet
function iloRemoveImportFromUserStylesheet(fileName) {
    var fileContents = iloReadUserChromeFile(fileName);
    var re = new RegExp("^[ \t]*@import.*bonus.ilo.css.*(\n)?$", "mg");

    if(re.test(fileContents)) {
        fileContents = fileContents.replace(re, '');
        var ret = iloWriteUserChromeFile(fileName, fileContents);
        return (ret == fileContents.length)
    } else {
        return true;
    }
}

//~ Adding CSS rules in 'bonus.ilo.css'.
function iloAddRulesToBonusCss(fontfamily, fontsize) {

    fontfamily = fontfamily ? '"' + fontfamily + '"' : 'inherit';
    fontsize = fontsize ? fontsize + 'px' : 'inherit';
    
    var fileContents = "/* ImgLikeOpera bonus for userChrome.css */\n" +
                       "*[ilo-ph-fix] {\n" +
                       "font-family: " + fontfamily + " !important;\n" +
                       "font-size: " + fontsize + " !important;\n" +
                       "font-weight:normal !important;\n" +
                       "line-height:140% !important;\n" +
                       "white-space:normal !important;\n" +
                       "}\n" +
                       "*[ilo-ph-fix='tofix'] {\n" +
                       "border:1px inset !important;\n" +
                       "padding:1px 2px !important;\n" +
                       "white-space:pre !important;\n" +
                       "}\n";
    
    var ret = iloWriteUserChromeFile('bonus.ilo.css', fileContents);
    return (ret == fileContents.length);
}

//~ Get values (font-family, font-size) from 'bonus.ilo.css'.
function iloReadRulesFromBonusCss() {
    var fileContents = iloReadUserChromeFile('bonus.ilo.css');
    
    if (!fileContents) return false;
    
    var match, rules=[];
    var reFFamily = new RegExp("family: \"(.*)\" !important", "i");
    var reFSize = new RegExp("size: (.*)px !important", "i");
    
    match = reFFamily.exec(fileContents);
    var fontfamily = (match == null) ? 'inherit' : match[1];
    
    match = reFSize.exec(fileContents);
    var fontsize = (match == null) ? 'inherit' : match[1];
    
    return [fontfamily, fontsize];
}

function iloCheckBonusUserCss()
{
    //~ check our little bonus css file.
    if (!iloReadRulesFromBonusCss())
        iloAddRulesToBonusCss();
    
    //~ check @import rule in 'userContent.css'.
    if (!iloUserStyleSheetHasImport())
        iloAddImportToUserStylesheet("userContent.css");
}